@@ -1,8 +1,8 @@
 /* redefine FD_SET */
-/* $OpenLDAP: pkg/ldap/include/ac/fdset.h,v 1.2.2.1 2004/07/16 18:35:04 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2013 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -22,6 +22,10 @@
 #ifndef _AC_FDSET_H
 #define _AC_FDSET_H
 
+#if !defined( OPENLDAP_FD_SETSIZE ) && !defined( FD_SETSIZE )
+#  define OPENLDAP_FD_SETSIZE 4096
+#endif
+
 #ifdef OPENLDAP_FD_SETSIZE
     /* assume installer desires to enlarge fd_set */
 #  ifdef HAVE_BITS_TYPES_H
